import React, { useState, useMemo, useEffect } from 'react'
import { Modal, Input, Table, Tooltip, message } from 'antd'
import { SnippetsOutlined } from '@ant-design/icons'
import { GetAccountConfigInfo, GetAccountPageList, getStationListByUserID } from '../../../../apis/process'
import styles from './index.less'
import { isJson } from '../../../../utils'
import Drag from '../../../components/Drag'

const AccountSelector = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { value, onChange, schema, addons } = props
  const { disabled, accountName, fieldshine, siteFilter, sql, isMultiple, presetValue, placeholder } = schema

  const [params, setParams] = useState({ info: '', total: 0, pageIndex: 1, pageSize: 20 })
  const [loading, setLoading] = useState(false)
  const [dataSource, setDataSource] = useState([])
  const [pageConfig, setPageConfig] = useState({ webShowFieldGroup: '' })
  const [visible, setVisible] = useState(false)
  const [keys, setKeys] = useState([])

  const columns = useMemo(() => {
    const { webShowFieldGroup } = pageConfig
    let fields = webShowFieldGroup.split(',')
    let array = []
    if (fields.length) {
      fields.forEach(v => {
        if (v) {
          array.push({
            key: new Date().getTime(),
            title: v,
            dataIndex: v,
            width: 120,
            ellipsis: true,
            render: (value, r) => <Tooltip placement="top" title={value}>{value}</Tooltip>
          })
        }
      })
    }
    return array
  }, [pageConfig])

  const onSearch = (value) => {
    getPageList({ info: value })
  }

  const iconClick = () => {
    if (addons) {
      if (!accountName) {
        return message.info('请配置台账名称！')
      }
      if (!fieldshine.length) {
        return message.info('请配置映射字段！')
      }
      setVisible(true)
      if (!pageConfig.webShowFieldGroup) {
        getPageConfig()
      }
      getPageList()
    }
  }

  const getPageConfig = async () => {
    const { code, data } = await GetAccountConfigInfo(accountName)
    if (code == 0) {
      setPageConfig(data)
    }
  }

  const getPageList = async (outParams = {}) => {
    let queryWheres = []
    if (siteFilter) {
      const { code, data } = await getStationListByUserID(...[userID, false])
      if (code === 0) {
        queryWheres = [
          {
            field: '所属站点',
            type: '包括',
            value: data.join(',')
          }
        ]
      }
    }
    setLoading(true)
    let param = {
      accountName: accountName,
      condition: sql ? window.btoa(encodeURIComponent(sql)) : '',
      sortFields: '录入时间',
      queryWheres: queryWheres,
      ...params,
      ...outParams,
    }
    const { data, code, msg } = await GetAccountPageList(param)
    if (code === 0) {
      if (data.jsonData) {
        let dataSource = JSON.parse(data.jsonData)
        setDataSource(dataSource)
        let valueArr = value ? value.split(',') : []
        let rows = dataSource.filter(v => valueArr.includes(v[fieldshine[0].fromField]))
        setKeys(rows.map(v => v.ID))
        setParams({ ...params, ...outParams, total: data.totalCount })
      }
    } else {
      message.error(msg)
    }
    setLoading(false)
  }

  const tableChange = ({ current, pageSize }) => {
    getPageList({ pageIndex: current, pageSize })
  }

  const rowChange = (keys, rows) => {
    setKeys(keys)
  }

  const onOk = () => {
    if (fieldshine.length > 1) {
      let row = dataSource.find(v => v.ID === keys[0]) || {}
      Object.keys(addons.formData).forEach(child => {
        Object.keys(addons.formData[child]).forEach(sun => {
          fieldshine.forEach(v => {
            if (sun === v.toField) {
              const schema = addons.getSchemaByPath(`${child}.${sun}`)
              if (schema.widget === 'fileUpload') {
                addons.setSchemaByPath(`${child}.${sun}`, { ...schema, otherFile: row[v.fromField] })
              } else {
                addons.setValue(`${child}.${sun}`, row[v.fromField] || '')
              }
            }
          })
        })
      })
    } else {
      let array = dataSource.filter(v => keys.includes(v.ID))
      if (array.length) {
        let rowValues = array.map(v => v[fieldshine[0].fromField]).filter(v => v).join(',')
        onChange(rowValues)
      }
    }
    setVisible(false)
  }

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  const addonAfter = <SnippetsOutlined onClick={() => iconClick()} style={{ color: 'rgba(0, 0, 0, 0.25)' }} />

  return (
    <div className={styles.AccountSelector}>
      <Input
        placeholder={!disabled ? placeholder : null}
        value={value}
        onClick={() => iconClick()}
        addonAfter={!disabled ? addonAfter : null}
        disabled={disabled}
      />
      <Drag
        onCancel={() => setVisible(false)}
        onOk={onOk}
        width='60%'
        title={accountName}
        visible={visible}
        getContainer={false}
        bodyStyle={{ height: 570, overflowY: 'auto' }}
      >
        <div style={{ display: 'flex' }}>
          <div style={{ lineHeight: '31px', paddingRight: '10px' }}>快捷搜索:</div>
          <Input.Search
            value={params.info}
            placeholder="请输入检索内容"
            allowClear
            onChange={(e) => setParams({ ...params, info: e.target.value })}
            onSearch={onSearch}
            enterButton
            size={'middle'}
            style={{ width: '400px', marginBottom: '10px' }}
          />
        </div>
        <Table
          rowKey={r => r.ID}
          loading={loading}
          bordered
          size="small"
          scroll={{ x: 1500, y: 400 }}
          rowSelection={{
            type: isMultiple && fieldshine.length === 1 ? 'checkbox' : 'radio',
            selectedRowKeys: keys,
            fixed: 'left',
            onChange: rowChange
          }}
          pagination={{
            total: params.total,
            current: params.pageIndex,
            pageSize: params.pageSize,
            showTotal: (total) => `共 ${total} 条`
          }}
          onChange={tableChange}
          dataSource={dataSource}
          columns={columns}
        />
      </Drag>
    </div>
  )

}

export default AccountSelector