import React, { useEffect, useState } from 'react'
import styles from './index.less'
import { Upload, Button, message, Modal } from 'antd'
import { UploadOutlined, FileOutlined, ArrowDownOutlined } from '@ant-design/icons'
import FileViewer from 'react-file-viewer'
import { uploadFileUrl, downloadFileUrl, downloadFile } from '../../../../apis/process'
import { filenameVerification } from '../../../../utils'
import Drag from '../../../components/Drag'

const accepts = {
  '全部': [],
  '图片': ['.bmp', '.gif', '.jpeg', 'tiff', '.png', '.svg', '.jpg'],
  '文档': ['.docx', '.xlsx', '.pdf'],
  '视频': ['.mp4'],
  '音频': ['.mp3'],
}

const getFileType = (fileName) => {
  if (fileName) {
    //图片
    if (fileName.includes('jpg')) {
      return 'jpg'
    }
    if (fileName.includes('png')) {
      return 'png'
    }
    if (fileName.includes('svg')) {
      return 'svg'
    }
    //文档
    if (fileName.includes('docx')) {
      return 'docx'
    }
    if (fileName.includes('xlsx')) {
      return 'xlsx'
    }
    if (fileName.includes('pdf')) {
      return 'pdf'
    }
    //音频
    if (fileName.includes('mp3')) {
      return 'mp3'
    }
    //视频
    if (fileName.includes('mp4')) {
      return 'mp4'
    }
  }
  return null
}

const FileUpload = (props) => {

  const { value, schema, onChange } = props
  const { disabled, fileType, presetValue, placeholder, preview, download } = schema

  const [showList, setShowList] = useState('')
  const [visible, setVisible] = useState(false)
  const [showFile, setShowFile] = useState({ fileType: '', filePath: '' })

  const showType = fileType === '图片' ? 'picture-card' : 'picture'

  const option = {
    name: 'file',
    action: `${window.location.origin}${uploadFileUrl}`,
    listType: showType,
    withCredentials: true,
    showUploadList: download ?
      {
        showDownloadIcon: '下载',
        downloadIcon: <ArrowDownOutlined style={{ color: 'white' }} />,
      } : {},
    beforeUpload(file, fileList) {
      /** @Tips: 解决提交文件中存在特殊字符的问题 */
      let _continueUpload = true;
      let _msg = {
        type: 'success',
        content: '上传成功！',
      };
      fileList.forEach(item => {
        let _msgObject = filenameVerification(item);
        if (_msgObject.type === 'error') {
          _continueUpload = false;
          _msg = {
            type: 'error',
            content: '上传失败！文件名不符合规则！',
          };
        }
      });
      _msg.type === 'error' ? message[_msg.type](_msg.content) : ''
      return _continueUpload;
    },
    onChange: ({ file, fileList, event }) => {
      // 检验名字，名字不通过不允许显示
      if (filenameVerification(file).type === 'error') return false;
      // 返回的链接在file.response内；不设置url，预览图表不可点击
      if (file.status === 'done' && file.response.code === 0) {
        file.url = `${downloadFileUrl}?filePath=${file.response.data}`
        file.sourcePath = file.response.data;
        message.success('上传成功！')
      } else if (file.status === 'done' && file.response.code !== 0) {
        file.status = 'error';
        message.error('上传失败！')
      }
      if (Array.isArray(fileList)) {
        let list = fileList.map(v => v.sourcePath || v.value)
        onChange(list.join(','))
        setShowList(JSON.stringify(fileList))
      } else {
        onChange('')
        setShowList()
      }
    },
    onPreview: (file) => {
      if (!preview) return message.info('该附件禁止预览')
      let fileType = getFileType(file.name)
      if (fileType) {
        setShowFile({ name: file.name, fileType: fileType, filePath: file.url })
        setVisible(true)
      }
    }
  }

  const iconRender = (file, listType) => {
    if (listType !== 'text') {
      let fileType = getFileType(file.name)
      if (fileType) {
        return <div className={styles.iconImg} type={fileType}></div>
      } else {
        return <div className={styles.iconImg} type={'通用'}></div>
      }
    }
    return <FileOutlined />
  }

  useEffect(() => {
    if (presetValue && !presetValue.includes('拍照相册')) {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (value) {
      let fileList = []
      let list = value ? value.split(',') : []
      list.forEach((item, index) => {
        if (item) { // @Tips: 直接过滤掉名字中有异常字符的文件
          let uid = index + '_' + Math.random()
          let _obj = {
            uid: uid,
            value: item,
            name: item.split('\\').reverse()[0],
            type: fileType === '图片' ? 'image/jpeg' : 'file',
            status: 'done',
            url: `${downloadFileUrl}?filePath=${item}`,
            sourcePath: item.url,
            originFileObj: { "uid": uid },
            "response": { "code": 0, "msg": "Ok", "data": item, "stackTrace": null },
            "xhr": {},
          };
          if (schema.renderTo === 'Image') _obj.thumbUrl = `${downloadFileUrl}?filePath=${item}`;
          fileList.push(_obj);
        }
      })
      setShowList(JSON.stringify(fileList))
    }
  }, [value])
  
  return (
    <div className={styles.uploadBox}>
      <Upload
        disabled={disabled}
        accept={accepts[fileType]}
        fileList={showList ? JSON.parse(showList) : []}
        className="upload-list-inline"
        iconRender={iconRender}
        {...option}
      >
        {
          showType !== 'picture-card' ?
            <Button icon={<UploadOutlined />}>{placeholder || '上传'}</Button>
            : (disabled ? null : (placeholder || '+ 上传'))
        }
      </Upload>
      <Drag
        width={'70%'}
        title={showFile.name}
        visible={visible}
        onCancel={() => setVisible(false)}
        bodyStyle={{ height: 650, overflowY: showFile.fileType.includes('xlsx') ? 'none' : "auto" }}
      >
        <FileViewer
          className='fileViewer'
          title='123'
          fileType={showFile.fileType}
          filePath={window.location.origin + showFile.filePath}
        />
      </Drag>
    </div>
  )

}

export default FileUpload